import serial
import crc16
import time

PAGE_SIZE_BYTES=64
MAX_PAGES=56

ser = serial.Serial('/dev/ttyUSB1', 115200, timeout=0.01)
ser.flushInput()
ser.flushOutput()
updateFile = open("bms-tiny.bin", "rb")
data = bytearray(updateFile.read())
updateFile.close()

numBytes = min(len(data), PAGE_SIZE_BYTES * MAX_PAGES)
numPages = (numBytes + PAGE_SIZE_BYTES - 1) / PAGE_SIZE_BYTES

while len(data) < (PAGE_SIZE_BYTES * MAX_PAGES):
    data.append(0xff)


print ("File length is %d bytes/%d pages" % (numBytes, numPages))
print ("Entering update mode...")

ser.write("\ncmupdate 3\n")
time.sleep(1)
page = 0

def sendPage(ser, data, page):
    binData = bytearray(PAGE_SIZE_BYTES + 3)
    binData[0] = page
    
    crc = crc16.crc16xmodem(chr(binData[0]))
    
    for i in range(0, PAGE_SIZE_BYTES):
        binData[i+1] = data[page * PAGE_SIZE_BYTES + i]
        crc = crc16.crc16xmodem(chr(binData[i+1]), crc)
    
    binData[1 + PAGE_SIZE_BYTES] = crc & 0xff
    binData[2 + PAGE_SIZE_BYTES] = (crc >> 8)

    print ("Sending page %d, crc %x" % (page, crc))
    
    for i in range(0, PAGE_SIZE_BYTES + 3):
        ser.write(chr(binData[i]))
        
    return crc

while page < MAX_PAGES:
    crc = sendPage(ser, data, page)
    
    # Allow 110ms for the page to be sent to the CMUs, 40ms
    # for the page to be written to flash and 20ms for the MMU replies
    time.sleep(0.17)
    response = ser.read(100)
    print response
    
    #if "ERR" not in response:
    #    page = page + 1
    if response == "":
        page = page + 1
        

# 0xaa in page field indicates end of update
#for i in range(0, PAGE_SIZE_BYTES + 6):
#    ser.write(chr(0xaa))
